package com.yerihyo.yeritools.collections;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import com.yerihyo.yeritools.debug.YeriDebug;

public class MapToolkit {
	public static class CollectionMap<K, V> extends CompareByEqualsMap<K,Collection<V>>{
		private static final long serialVersionUID = 1L;

		@SuppressWarnings("unchecked")
		public Collection<V> get(Object kObject){
			K k = null;
			try{ k = (K)kObject; }
			catch(ClassCastException ex){
				return null;
			}
			
			Collection<V> collection = super.get(k);
			if(collection==null){
				collection = new HashSet<V>();
				
				this.put(k, collection);
			}
			return collection;
		}
		public void putItem(K k, V v){
			Collection<V> collection = this.get(k);
			collection.add(v);
		}
	}
	public static class ListValueMap<K,V> extends HashMap<K,List<V>>{
		private static final long serialVersionUID = 1L;

		public List<V> get(K k){
			List<V> vList = super.get(k);
			if(vList==null){
				vList = new ArrayList<V>();
				super.put(k, vList);
			}
			return vList;
		}
		
		public V add(K k, V v){
			List<V> vList = this.get(k);
			vList.add(v);
			return v;
		}
		public Collection<? extends V> putAll(K k, Collection<? extends V> vCollection){
			List<V> vList = this.get(k);
			vList.addAll(vCollection);
			return vCollection;
		}
		
		public void putAll(Map<? extends K, ? extends List<V>> map){
			for(Map.Entry<? extends K, ? extends List<V>> entry : map.entrySet()){
				K k = entry.getKey();
				List<V> vList = this.get(k);
				vList.addAll(entry.getValue());
			}
//			return vList;
		}
		
		public boolean containsElement(V v){
			for(List<V> vList : this.values()){
				if(vList.contains(v)){
					return true;
				}
			}
			return false;
		}
		public Collection<V> valueElements(){
			List<V> returnValue = new ArrayList<V>();
			for(List<V> vList: super.values()){
				returnValue.addAll(vList);
			}
			return returnValue;
		}
	}
	
	public static void main(String[] args){
		test01();
	}
	private static void test01(){
		MapValueMap<Character,Integer,Integer> map = new MapValueMap<Character,Integer,Integer>();
		
		for(char c ='a'; c<='z'; c++){
			for(int i=0; i<10; i++){
				map.putItem(c, i, c+i);
			}
		}
		
		for(char c ='a'; c<='z'; c++){
			for(int i=0; i<10; i++){
				System.out.println(map.getItem(c, i));
			}
		}
		System.out.println(map.getItem('a', 14));
	}
	
	public static class MapValueMap<K1,K2,V> extends HashMap<K1,Map<K2,V>>{
		private static final long serialVersionUID = 1L;

		public Map<K2,V> get(Object key1){
			K1 k1;
			try{ k1 = (K1)key1; }
			catch(ClassCastException ex){ return null; }
			
			Map<K2,V> vMap = super.get(k1);
			if(vMap==null){
				vMap = new HashMap<K2,V>();
				super.put((K1)k1, vMap);
			}
			return vMap;
		}
		
		@SuppressWarnings("unchecked")
		public V getItem(Object key1, Object key2){
			K1 k1;
			try{ k1 = (K1)key1; }
			catch(ClassCastException ex){ return null; }
			
			Map<K2,V> vMap = super.get(k1);
			if(vMap==null){ return null; }
			
			return vMap.get(key2);
		}
		
		public V putItem(K1 k1, K2 k2, V v){
			Map<K2,V> vMap = this.get(k1);
			vMap.put(k2,v);
			return v;
		}
		public Collection<? extends V> addAll(K1 k1, K2 k2, Collection<? extends V> vCollection){
			Map<K2,V> vMap = this.get(k1);
			for(V v : vCollection){
				vMap.put(k2, v);
			}
			return vCollection;
		}
		
		public void putAll(Map<? extends K1, ? extends Map<K2,V>> map){
			for(Map.Entry<? extends K1, ? extends Map<K2,V>> entry : map.entrySet()){
				K1 k1 = entry.getKey();
				Map<K2,V> vMap = this.get(k1);
				for(Map.Entry<K2,V> subEntry : vMap.entrySet()){
					this.putItem(k1, subEntry.getKey(), subEntry.getValue());
				}
			}
//			return vList;
		}
		
		public boolean containsElement(V v){
			for(Map<K2,V> vMap : this.values()){
				if(vMap.containsValue(v)){
					return true;
				}
			}
			return false;
		}
		public Collection<V> valueElements(){
			List<V> returnValue = new ArrayList<V>();
			for(Map<K2,V> vMap : super.values()){
				returnValue.addAll(vMap.values());
			}
			return returnValue;
		}
	}
	
	public static class MapValueCountMap<K1,K2> extends MapValueMap<K1,K2,Integer>{
		private static final long serialVersionUID = 1L;
		
		public void increCount(K1 k1, K2 k2){ addCount(k1,k2,1); }
		public void addCount(K1 k1, K2 k2, int c){
			Integer count = super.getItem(k1, k2);
			super.putItem(k1, k2, count==null?c:count+c);
		}
	}
	
	public static class CompareByEqualsMap<K,V> extends HashMap<K,V> {
		private static final long serialVersionUID = 1L;
		private List<K> keyList = new ArrayList<K>();
		
		
		public List<K> keyList(){
			return this.keyList;
		}
		
		public K getRealKey(Object key){
			if(key==null){ return null; }
			for(K k : this.keySet()){
				if(key==k){ return k; }
				if(key.equals(k)){
					return k;
				}
			}
			return null;
		}
		
		@Override
		public boolean containsKey(Object key){
			return getRealKey(key)!=null;
		}
		
		@Override
		public boolean containsValue(Object value){
			for(V v : this.values()){
				if(v==null){
					if(value==null){
						return true;
					}else{
						continue;
					}
				}
				
				if(v.equals(value)){
					return true;
				}
			}
			return false;
		}
		
		@Override
		public V get(Object key){
			return super.get(getRealKey(key));
		}
		
		@Override
		public V remove(Object key){
			return super.remove(getRealKey(key));
		}
	}
	
	public static class CompareByEqualsTreeMap<K,V> extends TreeMap<K,V> {
		private static final long serialVersionUID = 1L;
		private List<K> keyList = new ArrayList<K>();
		
		
		public CompareByEqualsTreeMap(
				Comparator<? super K> c) {
			super(c);
		}

		public List<K> keyList(){
			return this.keyList;
		}
		
		public K getRealKey(Object key){
			if(key==null){ return null; }
			for(K k : this.keySet()){
				if(key==k){ return k; }
				if(key.equals(k)){
					return k;
				}
			}
			return null;
		}
		
		@Override
		public boolean containsKey(Object key){
			return getRealKey(key)!=null;
		}
		
		@Override
		public boolean containsValue(Object value){
			for(V v : this.values()){
				if(v==null){
					if(value==null){
						return true;
					}else{
						continue;
					}
				}
				
				if(v.equals(value)){
					return true;
				}
			}
			return false;
		}
		
		@Override
		public V get(Object key){
			return super.get(getRealKey(key));
		}
		
		@Override
		public V remove(Object key){
			return super.remove(getRealKey(key));
		}
	}
	
	public static class ListedKeyMap<K,V> extends CompareByEqualsMap<K,V> {
		private static final long serialVersionUID = 1L;
		private List<K> keyList = new ArrayList<K>();
		
		
		public List<Map.Entry<K,V>> entryList(){
			List<Map.Entry<K,V>> entryList = new ArrayList<Map.Entry<K,V>>();
			for(K k : keyList()){
				V v = this.get(k);
				entryList.add(new AbstractMap.SimpleEntry<K,V>(k,v));
			}
			return entryList;
		}
		
		public K getKey(int index){
			return keyList.get(index);
		}
		public V getValue(int index){
			return super.get(this.getKey(index));
		}
		
		public List<K> keyList(){
			return this.keyList;
		}
		
		public V put(K k, V v){
			keyList.add(k);
			return super.put(k, v);
		}
		public void putAll(Map<? extends K,? extends V> m){
			Set<? extends K> set = m.keySet();
			keyList.addAll(set);
			super.putAll(m);
		}
		public void putAll(ListedKeyMap<? extends K,? extends V> m){
			List<? extends K> set = m.keyList();
			keyList.addAll(set);
			super.putAll(m);
		}
		
		public Collection<V> values(){
			List<V> values = new ArrayList<V>();
			for(K k : this.keyList){
				values.add(this.get(k));
			}
			return values;
		}
	}

	public static <K,V> CharSequence toHTMLTableString(Map<K,V> map) {
		StringBuilder builder = new StringBuilder();
		
		StringBuilder keyRow = new StringBuilder();
		StringBuilder valueRow = new StringBuilder();
		keyRow.append("<TR>");
		valueRow.append("<TR>");
		
		for(K k : map.keySet()){
			keyRow.append("<TD>").append(k.toString()).append("</TD>");
			V v = map.get(k);
			valueRow.append("<TD>").append(v==null?"null":v.toString()).append("</TD>");
		}
		keyRow.append("</TR>");
		valueRow.append("</TR>");
		
		builder.append("<TABLE border='1'>");
		builder.append(keyRow);
		builder.append(valueRow);
		builder.append("</TABLE>");
		return builder;
		
	}
	public static <K,V> List<V> getValueList(Map<? extends K, ? extends V> map,
			Collection<? extends K> keyCollection) {
		List<V> vList = new ArrayList<V>();
		for(K k : keyCollection){
			V v = map.get(k);
			YeriDebug.ASSERT(v!=null);
			vList.add(v);
		}
		return vList;
	}
}
