package com.yerihyo.yeritools.collections;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.yerihyo.yeritools.math.StatisticsToolkit;

public class ValueMap<K> extends HashMap<K,Double> {
	public static final long serialVersionUID = 1;
	
	public ValueMap(){ super(); }
	public ValueMap(int initialCapacity){ super(initialCapacity); }
	public ValueMap(int initialCapacity, float loadFactor){ super(initialCapacity, loadFactor); }
	public ValueMap(Map<K,Double> map){ super(map); }
	
	public Double get(Object o){
		Double value = super.get(o);
		return value==null?0:value;
	}
	
	public Map<K,Double> getProportionMap(){
		Map<K,Double> returnMap = new HashMap<K,Double>();
		double totalValue = this.getTotalValue();
		
		for(Map.Entry<K, Double> entry : this.entrySet()){
			returnMap.put(entry.getKey(), (entry.getValue())/totalValue);
		}
		return returnMap;
	}
	
	public double getTotalValue(){
		double totalValue = 0;
		for(Map.Entry<K, Double> entry : this.entrySet()){
			totalValue += entry.getValue();
		}
		return totalValue;
	}
	
	public void addValue(Map.Entry<K,? extends Number> entry){
		this.addValue(entry.getKey(), entry.getValue().doubleValue());
	}
	public void addValue(K k, double d){
		double prevValue = 0;
		if(this.containsKey(k)){
			prevValue = this.get(k);
			this.remove(k);
		}
		this.put(k, prevValue+d);
	}
	
	public void addValueAll(Map<? extends K, ? extends Number> map){
		for(Map.Entry<? extends K,? extends Number> entry : map.entrySet()){
			K key = entry.getKey();
			double d = entry.getValue().doubleValue();
			this.addValue(key,d);			
		}
	}
	
	public double getValue(K k){ return this.get(k); }
	
	public double entropy(){
		return StatisticsToolkit.getEntropy(getProportionMap().values().toArray(new Double[0]));
	}
	public ValueMap<K> copy(List<K> desiredList) {
		if(desiredList==null){
			return new ValueMap<K>(this);
		}
		
		ValueMap<K> valueMap = new ValueMap<K>();
		for(K k : desiredList){
			Double d = this.get(k);
			if(d==null){ valueMap.put(k, 0.0d); }
			else{ valueMap.put(k, d); }
		}
		return valueMap;
	}
}