package com.yerihyo.yeritools.text;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.Set;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.yerihyo.yeritools.collections.CollectionsToolkit;
import com.yerihyo.yeritools.debug.YeriDebug;
import com.yerihyo.yeritools.io.FileToolkit;


public class StringToolkit {
	// private static Pattern alphaPattern = Pattern.compile("(\\p{Alpha})*");

	public static File windowsFontFolder = new File("C:/WINDOWS/Fonts");
	
	protected static void test04(){
		String content = FileToolkit.readFrom(new File("C:/testw.txt")).toString();
		String[] contentArray = content.split(StringToolkit.newLine());
		int totalLength = 0;
		
		int lineLength=0;
		for(int i=0; i<contentArray.length; i++){
			String oneContent = contentArray[i];
			int oneLength = oneContent.length();
//			System.out.println("["+oneContent+"],");
			if(i%2==0){
				lineLength = oneLength;
			}else{
				lineLength += oneLength;
				System.out.print(lineLength+",");	
			}
			totalLength += oneLength;
		}
//		content = content.replaceAll("[\n]", "");
		System.out.println(totalLength);
	}
	
	public static class StringComparator implements Comparator<String>{
		private boolean ignoreCase = false;
		
		public StringComparator(){}
		public StringComparator(boolean ignoreCase){ this.ignoreCase = ignoreCase; }
		
		@Override
		public int compare(String o1, String o2) {
			if(o1==null){
				if(o2==null){ return 0; }
				else{ return -1; }
			}
			
			if(ignoreCase){
				return o1.compareToIgnoreCase(o2);
			}else{
				return o1.compareTo(o2);
			}
		}
		public boolean isIgnoreCase() {
			return ignoreCase;
		}
		public void setIgnoreCase(boolean ignoreCase) {
			this.ignoreCase = ignoreCase;
		}
	}
	
	public static boolean isRomanCharacterOnly(CharSequence charSequence) {
		return Pattern.matches("(\\p{Alpha})*", charSequence);
	}
	
	public static int[] parseVersionString(String versionString){
		String[] versionStringArray = versionString.split("\\.");
		int[] versionNumber = new int[versionStringArray.length];
		for(int i=0; i<versionNumber.length; i++){
			versionNumber[i] = Integer.parseInt(versionStringArray[i]);
		}
		return versionNumber;
	}
	public static int[][] parseVersionString(String[] versionStringArray){
		int[][] returnValue = new int[versionStringArray.length][];
		for(int i=0; i<versionStringArray.length; i++){
			returnValue[i] = parseVersionString(versionStringArray[i]);
		}
		return returnValue;
	}
	
	protected static void test03(){
		String[] versionStringArray = new String[]{
				"12.1.4",
				"12.1.4.6",
				"12.1.4.5",
				"12.1.4.5.1",
				"12.1.4.6.2",
				"12.1.4.11",
		};
		int[][] versionArray = parseVersionString(versionStringArray);
		Arrays.sort(versionArray, new VersionComparator());
		for(int[] version : versionArray){
			System.out.println(CollectionsToolkit.toString(version));
		}
		
		
	
	}
	
	public static class VersionComparator implements Comparator<int[]>{

		@Override
		public int compare(int[] arg0, int[] arg1) {
			int minLength = Math.min(arg0.length, arg1.length);
			for(int i=0; i<minLength; i++){
				int diff = arg0[i]-arg1[i];
				if(diff!=0){ return diff; }
			}
			return arg0.length-arg1.length;
		}
		
	}

	public static boolean hasRomanCharacter(CharSequence charSequence) {
		for (int i = 0; i < charSequence.length(); i++) {
			char c = charSequence.charAt(i);
			if (isRomanCharacter(c)) {
				return true;
			}
		}
		return false;
	}

	public static void main(String[] args) {
		test06();
	}

	@SuppressWarnings("unused")
	private static void test01(){
		String[] testStringArray = new String[] { "adfa", "adlfjaofeijfaldskf",
				"AsfaidfAfadfaDFADfjiI", "JAKLDFJIELAKNEFLIAEWJF",
				"adfkaldsf adfkjaldsjf", "asldkjf9ealkdsfaldkfj",
				"14891248712984612849124", };

		for (String testString : testStringArray) {
			System.out.println(isRomanCharacterOnly(testString));
		}
	}

	public static String getInsideOfParentheses(String content,
			char startParenthesis, char endParenthesis) {
		int startIndex = content.indexOf(startParenthesis);
		if (startIndex == -1) {
			throw new IllegalArgumentException("No Start Parenthesis '"
					+ startParenthesis + "' in " + content);
		}

		int endIndex = content.lastIndexOf(endParenthesis);
		if (endIndex == -1) {
			throw new IllegalArgumentException("No Start Parenthesis '"
					+ endParenthesis + "' in " + content);
		}

		if (startIndex == endIndex) {
			throw new IllegalArgumentException("Index of '" + startParenthesis
					+ "' (" + startIndex + ") == Index of '" + endParenthesis
					+ "' (" + endIndex + ").");
		}
		return content.substring(startIndex + 1, endIndex);
	}

	public static CharSequence getArrayPrintString(double[] values) {
		DecimalFormat decimalFormat = new DecimalFormat("0.00");
		return getArrayPrintString(values, decimalFormat);
	}

	public static CharSequence getArrayPrintString(double[] values,
			DecimalFormat decimalFormat) {
		StringBuffer buffer = new StringBuffer();
		buffer.append('[');
		for (int i = 0; i < values.length; i++) {
			double value = values[i];
			if (i != 0) {
				buffer.append(", ");
			}

			buffer.append(decimalFormat.format(value));

		}
		buffer.append(']');
		return buffer;
	}

	public static double[] getMinMax(Collection<? extends String> c) {
		return getMinMax(c.toArray(new String[0]));
	}

	public static double[] getMinMax(String[] values) {
		if (values == null) {
			return null;
		}

		double[] minmax = new double[] { Double.POSITIVE_INFINITY,
				Double.NEGATIVE_INFINITY };
		for (String value : values) {
			if (value == null) {
				continue;
			}

			double doubleValue = 0;
			try {
				doubleValue = Double.parseDouble(value);
			} catch (NumberFormatException ex) {
				continue;
			}
			if (doubleValue < minmax[0]) {
				minmax[0] = doubleValue;
			}
			if (doubleValue > minmax[1]) {
				minmax[1] = doubleValue;
			}
		}
		return minmax;
	}

	public static interface StringAbbreviater {
		CharSequence abbreviate(Reader reader);

		CharSequence abbreviate(String s);
	}

	public static abstract class AbstractStringAbbreviater implements
			StringAbbreviater {
		@Override
		public CharSequence abbreviate(String s) {
			StringReader reader = new StringReader(s);
			return abbreviate(reader);
		}
	}

	public static class StringTrimmer extends AbstractStringAbbreviater {
		private int length = 10;
		private boolean doAddDots = true;

		public CharSequence abbreviate(Reader reader) {
			if (reader == null) {
				return null;
			}

			char[] buffer = new char[length + 1];
			int readLength = 0;
			try {
				readLength = reader.read(buffer, 0, length + 1);
			} catch (IOException ex) {
				return null;
			}
			StringBuffer stringBuffer = new StringBuffer();
			if (readLength > length) {
				stringBuffer.append(buffer, 0, length);
				if (doAddDots) {
					stringBuffer.append("...");
				}
			} else if (readLength > 0) {
				stringBuffer.append(buffer, 0, readLength);
			}

			return stringBuffer;
		}

		public int getLength() {
			return length;
		}

		public void setLength(int length) {
			this.length = length;
		}

		public boolean isDoAddDots() {
			return doAddDots;
		}

		public void setDoAddDots(boolean doAddDots) {
			this.doAddDots = doAddDots;
		}
	}

	public static String addSlashes(String s) {
		String t = s;
		t = t.replaceAll("\\\\", "\\\\");
		t = t.replaceAll("'", "\\'");
		t = t.replaceAll("\"", "\\\"");
		return t;
	}

	public static StringBuffer concatenate(String[] contents) {
		StringBuffer buffer = new StringBuffer();
		for (String s : contents) {
			buffer.append(s);
		}
		return buffer;
	}

	public static StringBuffer concatenate(List<String> contents) {
		StringBuffer buffer = new StringBuffer();
		for (String s : contents) {
			buffer.append(s);
		}
		return buffer;
	}
	
	public static StringBuffer concatenate(String[] contents, String deliminator) {
		StringBuffer buffer = new StringBuffer();
		for (int i=0; i<contents.length; i++) {
			if(i!=0){ buffer.append(deliminator); }
			String s = contents[i];
			buffer.append(s);
		}
		return buffer;
	}

	public static String getPrefix(String text, char deliminator) {
		int index = text.indexOf(deliminator);
		if (index < 0) {
			return text;
		}
		return text.substring(0, index);
	}

	public static String getSuffix(String text, char deliminator) {
		int index = text.indexOf(deliminator);
		return text.substring(index + 1);
	}

	private static String newLine = null;

	public static String getNewLine() {
		if (newLine == null) {
			newLine = System.getProperty("line.separator");
		}
		return newLine;
	}

	public static String newLine() {
		return getNewLine();
	}

	public static String getLineSeparator() {
		return getNewLine();
	}

	private static String fileSeparator = null;

	public static String getFileSeparator() {
		if (fileSeparator == null) {
			fileSeparator = System.getProperty("file.separator");
		}
		return fileSeparator;
	}

	public static int getLowerCharIndex(char c) {
		if (!(c >= 'a' && c <= 'z')) {
			throw new IllegalArgumentException(c
					+ " is not lower case alphabet character!");
		}
		return c - 'a';
	}
	
	
	public static Pattern punct = Pattern.compile("\\p{Punct}");
	
	public static String trimNonAlphaNumeric(String s) {
		int lastIndex = s.length()-1;
		for(;lastIndex>=0;lastIndex--){
			char c = s.charAt(lastIndex);
			if(Character.isLetterOrDigit(c)){
				break;
			}
		}
		return s.substring(0,lastIndex+1);
	}
	
	public static String trimNonAlphabet(String s) {
		int lastIndex = s.length()-1;
		for(;lastIndex>=0;lastIndex--){
			char c = s.charAt(lastIndex);
			if(Character.isLetter(c)){
				break;
			}
		}
		return s.substring(0,lastIndex+1);
	}

	public static boolean isRomanCharacter(char flag) {
		if (flag >= 'a' && flag <= 'z') {
			return true;
		}
		if (flag >= 'A' && flag <= 'Z') {
			return true;
		}
		return false;
	}

	private static char[] vowels = new char[] { 'a', 'e', 'i', 'o', 'u', };
	
	public static final String whiteSpacePatternString = "\\p{Space}+";
	public static final Pattern whiteSpacePattern = Pattern.compile(whiteSpacePatternString);
	
	public static final String newlinePatternString = "(\\r|\\n|\\r\\n)";
	public static final String newlinesPatternString = "("+newlinePatternString+")+";
	public static final Pattern newlinePattern = Pattern.compile(newlinePatternString);

	public static boolean isConsonant(String string) {
		if (string.length() > 1) {
			return false;
		}
		char c = string.charAt(0);
		if (!isRomanCharacter(c)) {
			return false;
		}
		return !CollectionsToolkit.contains(vowels, c);
	}

	public static boolean isVowel(String string) {
		if (string.length() > 1) {
			return false;
		}
		char c = string.charAt(0);
		if (!isRomanCharacter(c)) {
			return false;
		}
		return CollectionsToolkit.contains(vowels, c);
	}

	@SuppressWarnings("unused")
	private static void test02(){
		String[] patternArray = new String[]{
				"hello", "bye", "cool", "awesome",
		};
		String content = "Oh, Hello bob, this is awesome! I like it.";
		Pattern pattern = createPattern(patternArray, Pattern.CASE_INSENSITIVE);
		CharSequence result = addPaddingToSubstring(content, pattern, "<p>", "</p>");
		System.out.println(result);
	}
	public static CharSequence addPaddingToSubstring(CharSequence text,
			Pattern pattern, CharSequence padHeader, CharSequence padTail) {
		StringBuilder builder = new StringBuilder();
		Matcher matcher = pattern.matcher(text);

		int lastEndIndex = 0;
		while (matcher.find()) {

			int startIndex = matcher.start();
			int endIndex = matcher.end();

			builder.append(text.subSequence(lastEndIndex, startIndex));
			builder.append(padHeader);
			builder.append(text.subSequence(startIndex, endIndex));
			builder.append(padTail);
			lastEndIndex = endIndex;
		}
		builder.append(text.subSequence(lastEndIndex, text.length()));

		return builder;
	}
	public static CharSequence createRepeatPattern(int repeatCount, CharSequence sequence){
		StringBuilder builder = new StringBuilder();
		for(int i=0; i<repeatCount; i++){
			builder.append(sequence);
		}
//		return builder.substring(0,length);
		return builder.toString();
	}
	public static CharSequence filter(CharSequence text,
			Pattern pattern, CharSequence filler) {
		StringBuilder builder = new StringBuilder();
		Matcher matcher = pattern.matcher(text);

		int lastEndIndex = 0;
		while (matcher.find()) {

			int startIndex = matcher.start();
			int endIndex = matcher.end();

			
//			builder.append(text.subSequence(lastEndIndex, startIndex));
			builder.append(createRepeatPattern(startIndex-lastEndIndex, filler));
			builder.append(text.subSequence(startIndex, endIndex));
			
			lastEndIndex = endIndex;
		}
//		builder.append(text.subSequence(lastEndIndex, startIndex));
		builder.append(createRepeatPattern(text.length()-lastEndIndex, filler));

		return builder;
	}

	public static Pattern createPattern(String[] objectNameArray, int flags) {
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i < objectNameArray.length; i++) {
			String objectName = objectNameArray[i];
			if (i != 0) {
				builder.append('|');
			}
			builder.append('(').append(objectName).append(')');
		}
		Pattern pattern = Pattern.compile(builder.toString(), flags);
		return pattern;
	}


	public static enum EnumMatchType{
		CASE_INSENSITIVE;
		
		public String preprocess(String s){
			if(s==null){ return null; }
			
			if(this==CASE_INSENSITIVE){
				return s.toLowerCase();
			}else{
				throw new UnsupportedOperationException();
			}
		}
	}
	public static <T extends Enum<T>> int parseIndex(T[] values, String string, EnumMatchType enumMatchType) {
		if(string==null){ return -1; }
		
		String preprocessedString = enumMatchType.preprocess(string);
		for(int i=0; i<values.length; i++){
			T t = values[i];
			String tString = t.toString();
			String tPreprocessedString = enumMatchType.preprocess(tString);
			if(tPreprocessedString.equals(preprocessedString)){
				return i;
			}
		}
		return -1;
	}
	public static <T extends Enum<T>> T parse(T[] values, String string, EnumMatchType enumMatchType) {
		int index = parseIndex(values, string, enumMatchType);
		return values[index];
	}

	public static String addBufferFront(String s, char c, int length) {
		StringBuilder builder = new StringBuilder();
		int space = length - s.length();
		for(int i=0; i<space; i++){
			builder.append(c);
		}
		builder.append(s);
		return builder.toString();
	}
	public static String addBufferBack(String s, char c, int length) {
		StringBuilder builder = new StringBuilder();
		int space = length - s.length();
		builder.append(s);
		for(int i=0; i<space; i++){
			builder.append(c);
		}
		return builder.toString();
	}

	public static int newLineLength() {
		int newLineLength = StringToolkit.newLine().length();
		return newLineLength;
	}

	public static int getCharCount(File file) {
		try {
			BufferedReader reader = new BufferedReader(new FileReader(file));
			char[] buffer = new char[1024];
			int totalLength = 0;
			for(int readLength; (readLength=reader.read(buffer))>=0;){
				totalLength += readLength;
			}
			reader.close();
			return totalLength;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	public static int getLevenshteinDistance (String s, String t) {
		  if (s == null || t == null) {
		    throw new IllegalArgumentException("Strings must not be null");
		  }
				
		  /*
		    The difference between this impl. and the previous is that, rather 
		     than creating and retaining a matrix of size s.length()+1 by t.length()+1, 
		     we maintain two single-dimensional arrays of length s.length()+1.  The first, d,
		     is the 'current working' distance array that maintains the newest distance cost
		     counts as we iterate through the characters of String s.  Each time we increment
		     the index of String t we are comparing, d is copied to p, the second int[].  Doing so
		     allows us to retain the previous cost counts as required by the algorithm (taking 
		     the minimum of the cost count to the left, up one, and diagonally up and to the left
		     of the current cost count being calculated).  (Note that the arrays aren't really 
		     copied anymore, just switched...this is clearly much better than cloning an array 
		     or doing a System.arraycopy() each time  through the outer loop.)

		     Effectively, the difference between the two implementations is this one does not 
		     cause an out of memory condition when calculating the LD over two very large strings.  		
		  */		
				
		  int n = s.length(); // length of s
		  int m = t.length(); // length of t
				
		  if (n == 0) {
		    return m;
		  } else if (m == 0) {
		    return n;
		  }

		  int p[] = new int[n+1]; //'previous' cost array, horizontally
		  int d[] = new int[n+1]; // cost array, horizontally
		  int _d[]; //placeholder to assist in swapping p and d

		  // indexes into strings s and t
		  int i; // iterates through s
		  int j; // iterates through t

		  char t_j; // jth character of t

		  int cost; // cost

		  for (i = 0; i<=n; i++) {
		     p[i] = i;
		  }
				
		  for (j = 1; j<=m; j++) {
		     t_j = t.charAt(j-1);
		     d[0] = j;
				
		     for (i=1; i<=n; i++) {
		        cost = s.charAt(i-1)==t_j ? 0 : 1;
		        // minimum of cell to the left+1, to the top+1, diagonally left and up +cost				
		        d[i] = Math.min(Math.min(d[i-1]+1, p[i]+1),  p[i-1]+cost);  
		     }

		     // copy current distance counts to 'previous row' distance counts
		     _d = p;
		     p = d;
		     d = _d;
		  } 
				
		  // our last action in the above loop was to switch d and p, so p now 
		  // actually has the most recent cost counts
		  return p[n];
		}

	public static boolean isValidString(CharSequence key) {
		return key!=null && key.length()>0;
	}

	public static void assertEqualsIgnoreCase(String s1, String s2) {
		if(s1==null){
			YeriDebug.ASSERT(s2==null);
		}else{
			YeriDebug.ASSERT(s1.equalsIgnoreCase(s2), "'"+s1+" != '"+s2+"'");
		}
	}
	
	public static byte[] toByteArray(int i) {
		return new byte[] {
				(byte) (i >> 24 & 0xFF),
				(byte) (i >> 16 & 0xFF),
				(byte) (i >> 8 & 0xFF),
				(byte) (i & 0xFF),
				};
	}
	public static byte[] toByteArray(char c) {
		return new byte[] { (byte) (c & 0xFF), (byte) (c >> 8 & 0xFF), };
	}
	public static byte[] toByteArray(CharSequence str) {
		int length = str.length();
		byte[] byteArray = new byte[length * 2];
		for (int i = 0; i < length; i = i + 2) {
			char c = str.charAt(i);
			byteArray[i] = (byte) (c & 0xff);
			byteArray[i + 1] = (byte) (c >> 8 & 0xff);
		}
		return byteArray;
	}  
	
	protected static void test05() {
		for(byte b : toByteArray('a')){
			System.out.println(b);
		}
	}

	public static boolean equals(CharSequence text1, CharSequence text2) {
		if(text1==null){
			if(text2==null){ return true; }
			else{ return false; } 
		}
		else if(text2==null){ return false; }
		
		if(text1.length()!=text2.length()){ return false; }
		for(int i=0; i<text1.length(); i++){
			if(text1.charAt(i) != text2.charAt(i)){
				return false;
			}
		}
		return true;
	}

	public static boolean isLetterDigitOrDigitSign(char c) {
		if(Character.isLetterOrDigit(c)){ return true; }
		if(c=='-'){ return true; }
		if(c=='.'){ return true; }
		if(c==','){ return true; }
		return false;
	}

	public static int[] parseInt(String[] words) {
		int[] ints = new int[words.length];
		for(int i=0; i<words.length; i++){
			ints[i] = Integer.parseInt(words[i]);
		}
		return ints;
	}

	public static CharSequence toString(Collection<? extends Number> valueCollection) {
		StringBuilder builder = new StringBuilder();
		Iterator<? extends Number> iterator = valueCollection.iterator();
		for(int i=0; i<valueCollection.size(); i++){
			if(i!=0){
				builder.append(", ");
			}
			builder.append(iterator.next());
		}
		return builder;
	}

	public static CharSequence toString(int[] valueArray) {
		return toString(valueArray, null);
	}
	public static CharSequence toString(int[] valueArray, String[] brackets) {
		StringBuilder builder = new StringBuilder ();
		if(brackets!=null && brackets.length==2){
			builder.append(brackets[0]);
		}
		
		for(int i=0; i<valueArray.length; i++){
			if(i!=0){
				builder.append(", ");
			}
			builder.append(valueArray[i]);
		}
		if(brackets!=null && brackets.length==2){
			builder.append(brackets[1]);
		}

		return builder;
	}
	
	public static CharSequence toString(double[] valueArray) {
		return toString(valueArray, ",", null);
	}
	public static CharSequence toString(double[] valueArray, String delim, String[] brackets) {
		StringBuilder builder = new StringBuilder ();
		if(brackets!=null && brackets.length==2){
			builder.append(brackets[0]);
		}
		
		for(int i=0; i<valueArray.length; i++){
			if(i!=0){
				builder.append(delim+" ");
			}
			builder.append(valueArray[i]);
		}
		if(brackets!=null && brackets.length==2){
			builder.append(brackets[1]);
		}

		return builder;
	}

	public static boolean endsWithIgnoreCase(String value, String end) {
		String valueEnd = value.substring(value.length()-end.length(), value.length());
		return valueEnd.equalsIgnoreCase(end);
	}

	public static int[] getTokenEndIndexArray(CharSequence builder, Pattern pattern) {
		Matcher matcher = pattern.matcher(builder);
		Set<Integer> endIndexSet = new TreeSet<Integer>();
		
		while(matcher.find()){
			endIndexSet.add(matcher.end());
		}
		
		endIndexSet.add(builder.length());
		return CollectionsToolkit.toIntArray(endIndexSet);
	}
	private static void test06() {
		String s = "hello. my name is moonyoung kang.";
		System.out.println(CollectionsToolkit.toString(getTokenEndIndexArray(s, whiteSpacePattern)));
	}

	public static CharSequence replicate(String s, int count) {
		StringBuilder builder = new StringBuilder();
		for(int i=0; i<count; i++){
			builder.append(s);
		}
		return builder;
	}

//	public static String[] readFileToStringArrayByLine(File file) {
//		BufferedReader reader = new BufferedReader(file);
//		
//		return null;
//	}	

public static CharSequence toString(float[] valueArray) {
		return toString(valueArray, null);
	}
	public static CharSequence toString(float[] valueArray, String[] brackets) {
		StringBuilder builder = new StringBuilder ();
		if(brackets!=null && brackets.length==2){
			builder.append(brackets[0]);
		}
		
		for(int i=0; i<valueArray.length; i++){
			if(i!=0){
				builder.append(", ");
			}
			builder.append(valueArray[i]);
		}
		if(brackets!=null && brackets.length==2){
			builder.append(brackets[1]);
		}

		return builder;
	}

	
	private static String wordBoundaryPatternString = "\\b";
	public static String createWordPatternString(String[] nameArray) {
		StringBuilder builder = new StringBuilder();
		builder.append(wordBoundaryPatternString);
		builder.append("(");
		for(int i=0; i<nameArray.length; i++){
			if(i!=0){ builder.append("|"); }
			builder.append(nameArray[i]);
		}
		builder.append(")");
		builder.append(wordBoundaryPatternString);
		return builder.toString();
	}

	public static CharSequence generateRandomString(Random r, int length) {
		StringBuilder builder = new StringBuilder();
		for(int i=0; i<length; i++){
			int number = r.nextInt('z'-'a'+2);
			
			char c=0;
			if(number=='z'-'a'+1){ c=' '; }
			else{ c= (char)('a'+number); }
			builder.append(c);
		}
		return builder;
	}

	public static boolean isNumber(String annotationLabelString) {
		try{
			Double.parseDouble(annotationLabelString);
			return true;
		}catch(NumberFormatException ex){
			return false;
		}		
	}

	public static String[] toStringArray(double[] array) {
		String[] returnArray = new String[array.length];
		
		for(int i=0; i<array.length; i++){
			returnArray[i] = Double.toString(array[i]);
		}
		return returnArray;
	}

	public static CharSequence toString(String[] labelArray, String deliminator) {
		return toString(labelArray, deliminator, null);
	}
	public static CharSequence toString(String[] labelArray, String deliminator, String[] brackets) {
		if(labelArray==null || deliminator==null ){ return null; }
		if(brackets!=null){ YeriDebug.ASSERT(brackets.length>=2); }
		
		StringBuilder builder = new StringBuilder();
		for(int i=0; i<labelArray.length; i++){
			if(i!=0){ builder.append(deliminator); }
			if(brackets!=null){ builder.append(brackets[0]).append(labelArray[i]).append(brackets[1]); }
			else{ builder.append(labelArray[i]); }
		}
		return builder;
	}
	public static interface StringConverter<T>{ String getString(T t); }
	public static class DefaultStringConverter implements StringConverter<Object>{
		@Override
		public String getString(Object t) { return t.toString(); }
	}

	public static String[] toStringArray(int[] intArray) {
		String[] returnArray = new String[intArray.length];
		
		for(int i=0; i<intArray.length; i++){
			returnArray[i] = Integer.toString(intArray[i]);
		}
		return returnArray;
	}
}
