package com.yerihyo.yeritools.xml;


import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.sun.org.apache.xerces.internal.dom.DocumentImpl;
import com.sun.org.apache.xml.internal.serialize.OutputFormat;
import com.sun.org.apache.xml.internal.serialize.XMLSerializer;
import com.yerihyo.yeritools.collections.CollectionsToolkit;
import com.yerihyo.yeritools.debug.YeriDebug;
import com.yerihyo.yeritools.text.Base64;
import com.yerihyo.yeritools.text.StringToolkit;


public class XMLToolkit{
	public static void main(String[] args) throws FileNotFoundException, IOException, SAXException{
		test01();
	}
	
	public static void test01() throws IOException{
		Element e = null;
		Node n = null;
		// Document (Xerces implementation only).
		Document xmldoc= new DocumentImpl();
		// Root element.
		Element root = xmldoc.createElement("USERS");
		String[] id = {"PWD122","MX787","A4Q45"};
		String[] type = {"customer","manager","employee"};
		String[] desc = {"Tim@Home","Jack&Moud","John "};
		for (int i=0;i<id.length;i++)
		{
		  // Child i.
		  e = xmldoc.createElementNS(null, "USER");
		  e.setAttributeNS(null, "ID", id[i]);
		  e.setAttributeNS(null, "TYPE", type[i]);
		  n = xmldoc.createTextNode(desc[i]);
		  e.appendChild(n);
		  root.appendChild(e);
		}
		xmldoc.appendChild(root);
		System.out.println(xmlToString(root, utf8XMLOutputFormat));
	}
	
	public static OutputFormat utf8XMLOutputFormat = new OutputFormat("XML","UTF-8",true);
	public static OutputFormat isoXMLOutputFormat = new OutputFormat("XML","ISO-8859-1",true);
	
	
	public static void xmlToOutputStream(Node rootNode, OutputStream oStream, OutputFormat outputFormat) throws IOException{
		XMLSerializer serializer = new XMLSerializer(oStream,outputFormat);
		// As a DOM Serializer
		serializer.asDOMSerializer();
		serializer.serialize( rootNode );
	}
	
	public static String xmlToString(Node rootNode, OutputFormat outputFormat) throws IOException{
		ByteArrayOutputStream oStream = new ByteArrayOutputStream();
		xmlToOutputStream(rootNode, oStream, outputFormat);
		return oStream.toString();
		
//		StringWriter writer = new StringWriter();
//		XMLSerializer serializer = new XMLSerializer(writer,outputFormat);
//		// As a DOM Serializer
//		serializer.asDOMSerializer();
//		serializer.serialize( rootElement );
//		writer.close();
//		return writer.toString();
	}
	
	private static final int[] textNodeIDArray = new int[]{
		Node.TEXT_NODE, Node.CDATA_SECTION_NODE,
	};
	public static CharSequence getTextContent(Node node) {
		StringBuilder buffer = new StringBuilder();
		NodeList childList = node.getChildNodes();
		for (int i = 0; i < childList.getLength(); i++) {
			Node child = childList.item(i);
			int childNodeType = child.getNodeType();
			if(CollectionsToolkit.indexOf(textNodeIDArray, childNodeType)<0){
				continue;
			}
			buffer.append(child.getNodeValue());
		}
		return buffer.toString();
	}
	public static List<Element> getChildElements(Element root){
		List<Element> returnValue = new ArrayList<Element>();
		
		NodeList nodeList = root.getChildNodes();
		for(int i=0; i<nodeList.getLength(); i++){
			Node node = nodeList.item(i);
			if(!(node instanceof Element)){ continue; }
			returnValue.add((Element)node);
		}
		return returnValue;
	}
	public static Element getElementByAttributeValue(Element root, String tagName, String attrName, String attrValue){
		NodeList nodeList = root.getElementsByTagName(tagName);
		for(int i=0; i<nodeList.getLength(); i++){
			Element element = (Element)nodeList.item(i);
			if(element.getAttribute(attrName).equals(attrValue)){ return element; }
		}
		return null;
	}
	public static Element getFirstElement(Element root){
		NodeList nodeList = root.getChildNodes();
		for(int i=0; i<nodeList.getLength(); i++){
			Node node = nodeList.item(i);
			if(node instanceof Element){ return (Element)node; }
		}
		return null;
	}
	
	public static boolean hasChildElements(Element root){		
		NodeList nodeList = root.getChildNodes();
		for(int i=0; i<nodeList.getLength(); i++){
			if(nodeList.item(i) instanceof Element){ return true; }
		}
		return false;
	}
	
	public static Element getLastElementByName(Element element, String name){
		NodeList nodeList = element.getElementsByTagName(name);
		int listLength = nodeList.getLength();
		if(listLength==0) return null;
		return (Element)nodeList.item(listLength-1);
	}
	
	public static Element getLastChildElementByName(Element root, String name){
		if(name==null){ return null; }
		
		Element returnElement = null;
		NodeList nodeList = root.getChildNodes();
		for(int i=0; i<nodeList.getLength(); i++){
			Node node = nodeList.item(i);
			if(!(node instanceof Element)){ continue; }
			
			Element element = (Element)node;
			if( name.equals(element.getTagName()) ){
				returnElement = element;
			}
		}
		return returnElement;
	}
	
	public static Set<String> xmlOptionTypeToSet(
			Element root, String rootNodeTag,
			String childNodeTag)
			throws Exception{
		HashSet<String> set = new HashSet<String>();
		if(!root.getTagName().equalsIgnoreCase(rootNodeTag)) throw new Exception();
		NodeList children = root.getChildNodes();
				    
	    for(int i=0; i<children.getLength(); i++){
	    	Node node = children.item(i);
	    	if(node.getNodeType() != Node.ELEMENT_NODE) continue;
	    	
	    	Element optionElement = (Element)node;
	    	
	    	// if not 'option' tag, then continue
			try{ AssertTagName(optionElement, "xDoc", childNodeTag); }
			catch(Exception ex){ continue; }
			
			String attributeValue = XMLToolkit.getTextContent(optionElement).toString();
			set.add(attributeValue);
	    }
	    return set;		
	}
	
	public static Map<String, String> xmlAttributeTypeToMap(
			Element root, String rootNodeTag)
			throws Exception{
		HashMap<String, String> map = new HashMap<String, String>();
		if(!root.getTagName().equalsIgnoreCase(rootNodeTag)) throw new Exception();
		NamedNodeMap namedNodeMap = root.getAttributes();
				    
	    for(int i=0; i<namedNodeMap.getLength(); i++){
	    	Node node = namedNodeMap.item(i);
	    	if(node.getNodeType() != Node.ATTRIBUTE_NODE) continue;
	    	
	    	String key = node.getNodeName();
	    	String value = node.getNodeValue();
			map.put(key, value);
	    }
	    return map;		
	}
	
	public static CharSequence wrapStartBrackets(CharSequence cs){
		StringBuilder builder = new StringBuilder();
		builder.append('<').append(cs).append('>');
		return builder;
	}
	public static CharSequence wrapEndBrackets(CharSequence cs){
		StringBuilder builder = new StringBuilder();
		builder.append("</").append(cs).append('>');
		return builder;
	}
	public static CharSequence wrapStartEndBrackets(CharSequence cs){
		StringBuilder builder = new StringBuilder();
		builder.append('<').append(cs).append("/>");
		return builder;
	}
	public static CharSequence wrapContentWithTag(CharSequence content, CharSequence tag){
		StringBuilder builder = new StringBuilder();
		builder.append(wrapStartBrackets(tag));
		builder.append(content);
		builder.append(wrapEndBrackets(tag));
		return builder;
	}
	
	public static String mapToXMLAllContentForm(
			Map<String,String> map, String rootNodeTag,
			String optionNodeTag, String keyNodeTag, String valueNodeTag, boolean doCDATA)
			{
		StringBuilder builder = new StringBuilder();
		builder.append(wrapStartBrackets(rootNodeTag));
		
		for(Map.Entry<String, String> entry : map.entrySet()){
			builder.append(wrapStartBrackets(optionNodeTag));
			
			builder.append(wrapStartBrackets(keyNodeTag));
			CharSequence key = entry.getKey();
			if(doCDATA){ key = wrapCdata(key); }
			builder.append(key);
			builder.append(wrapEndBrackets(keyNodeTag));
			
			builder.append(wrapStartBrackets(valueNodeTag));
			CharSequence value = entry.getValue();
			if(doCDATA){ value = wrapCdata(value); }
			builder.append(value);
			builder.append(wrapEndBrackets(valueNodeTag));
			
			builder.append(wrapEndBrackets(optionNodeTag));
		}
		
		builder.append(wrapEndBrackets(rootNodeTag));
		return builder.toString();
	}
	
	public static Map<String, String> xmlAllContentFormToMap(
			Element root, String rootNodeTag,
			String optionNodeTag, String keyNodeTag, String valueNodeTag){
		HashMap<String, String> map = new HashMap<String, String>();
		StringToolkit.assertEqualsIgnoreCase(root.getTagName(), rootNodeTag);
		
		NodeList children = root.getChildNodes();
				    
	    for(int i=0; i<children.getLength(); i++){
	    	Node child = children.item(i);
	    	if(child.getNodeType() != Node.ELEMENT_NODE) continue;
	    	Element optionElement = (Element)child;
	    	
	    	// if not 'option' tag, then continue
			assertTagName(optionElement, optionNodeTag);
			
			NodeList optionChildren = optionElement.getChildNodes();
			
			CharSequence key = null;
			CharSequence value = null;
			for(int j=0; j<optionChildren.getLength(); j++){
				Node optionChild = optionChildren.item(j);
				if(optionChild.getNodeType() != Node.ELEMENT_NODE) continue;
				
				Element optionChildElement = (Element)optionChild;
				
				String optionChildElementTagName = optionChildElement.getTagName();
				if(optionChildElementTagName.equals(keyNodeTag)){
					key = XMLToolkit.getTextContent(optionChildElement);
				}
				else if(optionChildElementTagName.equals(valueNodeTag)){
					value = XMLToolkit.getTextContent(optionChildElement);
				}
			}
			
			if(StringToolkit.isValidString(key) && StringToolkit.isValidString(value)){
				map.put(key.toString(), value.toString());
			}
	    }
	    return map;		
	}
	
	public static Map<String, String> xmlOptionTypeToMap(
			Element root, String rootNodeTag,
			String childNodeTag, String attributeTag)
			throws Exception{
		HashMap<String, String> map = new HashMap<String, String>();
		if(!root.getTagName().equalsIgnoreCase(rootNodeTag)) throw new Exception();
		NodeList children = root.getChildNodes();
				    
	    for(int i=0; i<children.getLength(); i++){
	    	Node node = children.item(i);
	    	if(node.getNodeType() != Node.ELEMENT_NODE) continue;
	    	
	    	Element optionElement = (Element)node;
	    	
	    	// if not 'option' tag, then continue
			assertTagName(optionElement, childNodeTag);
			
			// if no 'name' attribute, then continue
			String attributeName = optionElement.getAttribute(attributeTag);
			if(attributeName.length()==0) continue;
			
			String attributeValue = XMLToolkit.getTextContent(optionElement).toString();
			map.put(attributeName, attributeValue);
	    }
	    return map;		
	}
	
	public static Map<String, String> xmlSimpleTypeToMap(
			Element root, String rootNodeTag){
		HashMap<String, String> map = new HashMap<String, String>();
		YeriDebug.ASSERT(root.getTagName().equalsIgnoreCase(rootNodeTag));
		
	    for(Element element : XMLToolkit.getChildElements(root)){
	    	String tagName = element.getTagName();
	    	CharSequence content = XMLToolkit.getTextContent(element);
	    	
			map.put(tagName, content.toString());
	    }
	    return map;		
	}
	
	public static <K,V> String mapToXMLSimpleTypeString(CharSequence rootNodeTag, Map<K,V> map){
		StringBuilder builder = new StringBuilder();
		builder.append("<").append(rootNodeTag).append(">").append(StringToolkit.newLine());
		
		for(Map.Entry<K, V> entry : map.entrySet()){
			builder.append("<").append(entry.getKey().toString()).append(">").append(StringToolkit.newLine());
			builder.append(entry.getValue().toString()).append(StringToolkit.newLine());
			builder.append("</").append(entry.getKey().toString()).append(">").append(StringToolkit.newLine());
		}
		
		builder.append("</").append(rootNodeTag).append(">").append(StringToolkit.newLine());
		return builder.toString();
	}
	
	/**
	 * <key><![CDATA[value]]></key> form
	 * @param thisEntry
	 * @param nodeTag
	 * @param attributeTag
	 * @return
	 */
	protected static String entryItemToXMLString(
			Map.Entry<String, String> thisEntry, String nodeTag, String attributeTag){
		StringBuffer returnString = new StringBuffer();
		
		String key = thisEntry.getKey();
		String value = thisEntry.getValue();
		
		returnString.append("<").append(nodeTag);
		returnString.append(attributeTag).append("=\"").append(key).append("\"");
		returnString.append(">");
		
		returnString.append(XMLToolkit.wrapCdata(value));
		
		returnString.append("<").append(nodeTag);
		returnString.append(attributeTag).append("=\"").append(key).append("\"");
		returnString.append("/>");
		
		return returnString.toString();
		
	}
	public static String mapToXmlOptionTypeString(
			Map<String, String> map,
			String rootNodeTag, String childNodeTag, String attributeTag)
			{
		StringBuffer returnString = new StringBuffer();
		returnString.append("<").append(rootNodeTag).append(">");
		
		for(Map.Entry<String, String> entry : map.entrySet()){
			returnString.append(
					entryItemToXMLString(entry, childNodeTag, attributeTag));
		}
		
		returnString.append("<").append(rootNodeTag).append("/>");				
		return returnString.toString();
	}
	
	public static void assertTagName (Element xDoc, String tagName) throws RuntimeException
	{
		if (!xDoc.getTagName().equalsIgnoreCase(tagName))
		{throw new RuntimeException ("Attempt to initialize with the wrong tag; (expected '<" + tagName + 
							  "/>', but got '<" + xDoc.getTagName() + "/>'");}
	}
	
	/**
	 * @deprecated
	 * @param xDoc
	 * @param className
	 * @param tagName
	 * @throws Exception
	 */
	public static void AssertTagName (Element xDoc, String className, String tagName) throws RuntimeException
	{
		if (!xDoc.getTagName().equalsIgnoreCase(tagName))
		{throw new RuntimeException ("Attempt to initialize " + className + 
							  " with the wrong tag; (expected '<" + tagName + 
							  "/>', but got '<" + xDoc.getTagName() + "/>'");}
	}
	
	public static Locale extractLocale(Element element){
		if(element==null){ return Locale.ENGLISH; }
		
		Attr language = element.getAttributeNode("language");
		if(language==null){ return Locale.ENGLISH; }
		String languageName = language.getNodeValue();
		
		Attr country = element.getAttributeNode("country");
		if(country==null){ return new Locale(languageName); }
		String countryName = country.getNodeValue();
		
		Attr variant = element.getAttributeNode("variant");
		if(variant==null){ return new Locale(languageName, countryName); }
		String variantName = country.getNodeValue();
		
		return new Locale(languageName, countryName, variantName);
	}
	
	protected static String toXMLAttributeString(
			Map.Entry<String, String> thisEntry, String nodeTag, String attributeTag){
		StringBuffer returnString = new StringBuffer();
		
		String key = thisEntry.getKey();
		String value = thisEntry.getValue();
		
		returnString.append("<").append(nodeTag);
		returnString.append(attributeTag).append("=\"").append(key).append("\"");
		returnString.append(">");
		
		returnString.append("<![CDATA[").append(value).append("]]>");
		
		returnString.append("</").append(nodeTag);
		returnString.append(attributeTag).append("=\"").append(key).append("\"");
		returnString.append(">");
		
		return returnString.toString();
		
	}
	
	protected static String toXMLString(Map.Entry<String, String> thisEntry){
		StringBuffer returnString = new StringBuffer();
		
		String key = thisEntry.getKey();
		String value = thisEntry.getValue();
		
		returnString.append("<").append(key).append(">");
		returnString.append("<![CDATA[").append(value).append("]]>");
		returnString.append("</").append(key).append(">");
		
		return returnString.toString();
		
	}
	public static String toXMLString(Map<String, String> map){
		StringBuffer returnString = new StringBuffer();
		
		Set<Map.Entry<String, String>> entrySet = map.entrySet();
		Iterator<Map.Entry<String, String>> iterator = entrySet.iterator();
		while(iterator.hasNext()){
			Map.Entry<String, String> thisEntry = iterator.next();
			returnString.append(toXMLString(thisEntry));
		}				
		return returnString.toString();
	}
	public static String toXMLAttributeString(
			Map<String, String> map,
			String rootNodeTag, String childNodeTag, String attributeTag)
			{
		StringBuffer returnString = new StringBuffer();
		returnString.append("<").append(rootNodeTag).append(">");
		
		Set<Map.Entry<String, String>> entrySet = map.entrySet();
		Iterator<Map.Entry<String, String>> iterator = entrySet.iterator();
		while(iterator.hasNext()){
			Map.Entry<String, String> thisEntry = iterator.next();
			returnString.append(
					toXMLAttributeString(thisEntry, childNodeTag, attributeTag));
		}
		
		returnString.append("<").append(rootNodeTag).append("/>");				
		return returnString.toString();
	}
	
	public static String getHeaderString() {
		return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
	}
	public static CharSequence wrapCdata(CharSequence cs) {
		StringBuilder builder = new StringBuilder();
		builder.append("<![CDATA[").append(cs).append("]]>");
		return builder;
	}

	public static String XMLVersionString(String version, String encodingString) {
		return "<?xml version=\""+version+"\" encoding=\""+encodingString+"\"?>";
	}

	public static CharSequence wrapSerializedContentWithTag(Serializable s, String xmlTag) {
		return XMLToolkit.wrapContentWithTag(XMLToolkit.wrapCdata( Base64.encodeObject(s) ), xmlTag);
	}
}
